@@ -1,10 +1,12 @@
 """bytes (de)serialization utilities."""
 __docformat__ = "google"
 
-from typing import Any
+from typing import Any, Optional
 
 from base64 import b64decode, b64encode
 
+from turbo_broccoli.environment import is_nodecode
+
 
 def _bytes_from_json_v1(dct: dict) -> bytes:
     """
@@ -13,12 +15,14 @@ def _bytes_from_json_v1(dct: dict) -> bytes:
     return b64decode(dct["data"])
 
 
-def from_json(dct: dict) -> bytes:
+def from_json(dct: dict) -> Optional[bytes]:
     """
     Deserializes a dict into a bytes object. See `to_json` for the
     specification `dct` is expected to follow. In particular, note that `dct`
     must contain the key `__bytes__`.
     """
+    if is_nodecode("bytes"):
+        return None
     DECODERS = {
         1: _bytes_from_json_v1,
     }
