@@ -54,6 +54,29 @@ except ModuleNotFoundError:
     HAS_PYTORCH = False
 
 
+try:
+    import turbo_broccoli.scipy
+
+    HAS_SCIPY = True
+except ModuleNotFoundError:
+    HAS_SCIPY = False
+
+try:
+    import turbo_broccoli.sklearn
+
+    HAS_SKLEARN = True
+except ModuleNotFoundError:
+    HAS_SKLEARN = False
+
+
+try:
+    import turbo_broccoli.bokeh
+
+    HAS_BOKEH = True
+except ModuleNotFoundError:
+    HAS_BOKEH = False
+
+
 class TurboBroccoliDecoder(json.JSONDecoder):
     """
     TurboBroccoli's custom JSON decoder class. See the README for the list of
@@ -67,8 +90,6 @@ class TurboBroccoliDecoder(json.JSONDecoder):
         """Deserialization hook"""
         DECODERS: Dict[str, Callable[[dict], Any]] = {
             "__bytes__": turbo_broccoli.bytes.from_json,
-            "__collections__": turbo_broccoli.collections.from_json,
-            "__dataclass__": turbo_broccoli.dataclass.from_json,
         }
         if HAS_KERAS:
             DECODERS["__keras__"] = turbo_broccoli.keras.from_json
@@ -82,6 +103,15 @@ class TurboBroccoliDecoder(json.JSONDecoder):
             DECODERS["__secret__"] = turbo_broccoli.secret.from_json
         if HAS_TENSORFLOW:
             DECODERS["__tensorflow__"] = turbo_broccoli.tensorflow.from_json
+        if HAS_SCIPY:
+            DECODERS["__scipy__"] = turbo_broccoli.scipy.from_json
+        if HAS_SKLEARN:
+            DECODERS["__sklearn__"] = turbo_broccoli.sklearn.from_json
+        if HAS_BOKEH:
+            DECODERS["__bokeh__"] = turbo_broccoli.bokeh.from_json
+        # Intentionally put last
+        DECODERS["__collections__"] = turbo_broccoli.collections.from_json
+        DECODERS["__dataclass__"] = turbo_broccoli.dataclass.from_json
         for t, f in DECODERS.items():
             if t in dct:
                 return f(dct)
@@ -98,9 +128,6 @@ class TurboBroccoliEncoder(json.JSONEncoder):
 
         ENCODERS: List[Callable[[Any], dict]] = [
             turbo_broccoli.bytes.to_json,
-            turbo_broccoli.collections.to_json,
-            turbo_broccoli.dataclass.to_json,
-            turbo_broccoli.generic.to_json,
         ]
         if HAS_KERAS:
             ENCODERS.append(turbo_broccoli.keras.to_json)
@@ -114,6 +141,18 @@ class TurboBroccoliEncoder(json.JSONEncoder):
             ENCODERS.append(turbo_broccoli.secret.to_json)
         if HAS_TENSORFLOW:
             ENCODERS.append(turbo_broccoli.tensorflow.to_json)
+        if HAS_SCIPY:
+            ENCODERS.append(turbo_broccoli.scipy.to_json)
+        if HAS_SKLEARN:
+            ENCODERS.append(turbo_broccoli.sklearn.to_json)
+        if HAS_BOKEH:
+            ENCODERS.append(turbo_broccoli.bokeh.to_json)
+        # Intentionally put last
+        ENCODERS += [
+            turbo_broccoli.collections.to_json,
+            turbo_broccoli.dataclass.to_json,
+            turbo_broccoli.generic.to_json,
+        ]
         for f in ENCODERS:
             try:
                 return f(o)
@@ -134,3 +173,13 @@ class TurboBroccoliEncoder(json.JSONEncoder):
             d = turbo_broccoli.collections._namedtuple_to_json(o)
             return super().encode({"__collections__": d})
         return super().encode(o)
+
+
+def from_json(doc: str) -> Any:
+    """Converts a JSON document back to a Python object."""
+    return json.loads(doc, cls=TurboBroccoliDecoder)
+
+
+def to_json(obj: Any) -> str:
+    """Converts an object to JSON."""
+    return json.dumps(obj, cls=TurboBroccoliEncoder)
